import argparse
import sys

from metadocker.common.errors import MetadockerError
from metadocker.db.db import settings


class Argument:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.parser = None

    def call(self, parser):
        self.parser = parser
        parser.add_argument(*self.args, **self.kwargs)

class ArgumentSubCommands(Argument):
    def __init__(self, name, commands, command_name, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = name
        self.commands = commands
        self.command_name = command_name
        self.subparsers = None

    def call(self, parser):
        self.parser = parser
        self.subparsers =  parser.add_subparsers(prog=self.name, parser_class=argparse.ArgumentParser)
        for k in self.commands:
            self._add_command(classe=k)



    def _add_command(self, classe):
        kwargs = {}
        if classe.HELP:
            kwargs["help"] = classe.HELP
        if classe.ALIASES:
            kwargs["aliases"] = classe.ALIASES

        parser = self.subparsers.add_parser(classe.NAME, **kwargs)
        for k in classe.ARGUMENTS:
            k.call(parser)
        parser.set_defaults(**{self.command_name:classe})
        return parser

class Command:
    NAME = None
    HELP = None
    ALIASES = []
    ARGUMENTS = []
    CATCH_EXCEPTION = True
    CONFIG_SET = True
    def __init__(self, parser, data):
        self._data = data
        self._parser = parser
        data = data.__dict__

        for k, v in data.items():
            setattr(self, k, v)

    def check(self, raise_exception=False):
        try:
            if self.CONFIG_SET:
                settings.validate()

        except MetadockerError as err:
            print("Erreur de configuration interne à métadocker", file=sys.stderr)
            print(err.get_error(), file=sys.stderr)
            if raise_exception:
                raise err
            return False
        return True


    def start(self):
        if self.check():
            try:
                return self.run()
            except MetadockerError as err:
                print("Erreur", file=sys.stderr)
                print(err.get_error(), file=sys.stderr)
                return -1

    def run(self):
        raise NotImplementedError()


class SpecificCommand(Command):


    def start(self, env):
        env.debug = self.debug
        if self.check():
            try:
                env.read()
                return self.run(env)
            except MetadockerError as err:
                print("Erreur", file=sys.stderr)
                print(err.get_error(), file=sys.stderr)
                return -1



