import sys
from pathlib import Path

from metadocker.cmdline.commands.base import Argument, Command
from metadocker.common.errors import MetadockerError
from metadocker.db.db import settings
from metadocker.register import load_env


class ConfigCommand(Command):
    NAME = "config"
    HELP = "Configure l'application\nEx metadocker config -l # liste les config\n" \
           "metadocker config my.key # renvoie la valeur de my.key\n" \
           "metadocker config \"my.key=my value\" # affctre la valeur \"my value\" à my.key"
    CONFIG_SET = False

    ARGUMENTS = [
        Argument("--list", "-l", action="store_true", help="Liste les configurations"),
        Argument("--short", "-s", action="store_true", help="N'affiche pas les clés"),
        Argument("--validate", "-v", action="store_true", help="Vérifie la configuration"),
        Argument("args",  nargs="*", help="Les ")
    ]

    def run(self):
        settings.load()
        vars = {x.name: x for x in settings._vars_}
        if  not self.args and not self.validate:
            for k in sorted(set(settings)|set(vars)):
                if k in vars:
                    var = vars[k]
                    if var.help:
                        help = var.help.replace('\n','\n# ')
                        print(f"# {k}: {help}")
                    if k in settings:
                        print(f"{k}={settings[k]}")
                    else:
                        print(f"{k} : Non défini")
                else:
                    print(f"{k}={k}")
            exit(0)

        for arg in self.args:
            if "=" in arg:
                key, _, value = arg.partition("=")
                if key in vars:
                    try:
                        settings[key] = vars[key].to_python(value)
                    except MetadockerError as err:
                        print(f"Erreur de saisie")
                        print(err.get_error())
                else:
                    settings[key] = value
            else:
                if arg in settings:
                    if self.short:
                        print(settings[arg])
                    else:
                        print(f"{arg}={settings[arg]}")
                else:
                    print(f"Clé {arg} introuvable", file=sys.stderr)

        if self.validate:
            if self.check(raise_exception=True):
                print(f"La configuration est valide")
