import argparse
import os

from metadocker.cmdline.commands.base import Argument, Command, ArgumentSubCommands
from metadocker.cmdline.commands import specific
from metadocker.db.db import settings
from metadocker.db.manager import DockerManager

class chdir:
    def __init__(self, path):
        self.old_path = None
        self.path = path

    def __enter__(self):
        self.old_path = os.getcwd()
        os.chdir(self.path)
    def __exit__(self, exc_type, exc_val, exc_tb):
        os.chdir(self.old_path)

class UseCommand(Command):
    NAME = "use"
    HELP = "Utilise le docker"
    ARGUMENTS = [
        Argument("name", help="Le nom du docker"),
        ArgumentSubCommands("parser", commands=[
            specific.build.BuildCommand,
            specific.start.StartCommand,
            specific.stop.StopCommand,
            specific.run.RunCommand,
            specific.rm.RmCommand,
            specific.status.StatusCommand,
            specific.logs.LogsCommand,
            specific.systemd.SystemdCommand,
            specific.shell.ShellCommand
            ], command_name="sub_command"
        )
    ]

    def run(self):
        docker = settings.app_docker_cache.get(self.name)
        parser = None
        for arg in self.ARGUMENTS:
            if isinstance(arg, ArgumentSubCommands) and arg.name == "parser":
                parser = arg.parser
                if not getattr(self, "sub_command", None):
                    arg.parser.print_help()
                    return -1

        env = docker.get_env_object()
        with chdir(docker.root_dir):
            return self.sub_command(parser, self._data).start(env)
