import shutil
import sys
from importlib import _bootstrap_external

from pathlib import Path

from metadocker.db.db import settings
from metadocker.docker.driver import driver
from metadocker.register import load_env

import sys
import importlib.util

class LoaderContext:

    def __init__(self, suffix):
        self.suffix = suffix
        self.old_fct = None
        self.env = None

        from metadocker.env.base import Env
        self._Env = Env

    def __enter__(self):
        loader = [x for x, y in _bootstrap_external._get_supported_file_loaders() if ".py" in y][0]
        self.old_fct = _bootstrap_external._get_supported_file_loaders
        _bootstrap_external._get_supported_file_loaders = lambda : [(loader, [self.suffix])]
        self._Env._suspend_run_ = True
        self._Env._current_env = None
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._Env._suspend_run_ = False
        _bootstrap_external._get_supported_file_loaders=self.old_fct
        self.env = self._Env._current_env

class DockerEntry:
    _get_state_bypass_ = ["_env", "_meta", "_env_holder"]
    _config_file_ = "docker.cfg"
    def __init__(self, name):
        self.name = name
        self.root_dir = settings.app_dir / self.name
        self.ports = None
        self._init()

    def _init(self):
        self._env  = False

    def _get_env(self):
        with LoaderContext(Path(self._config_file_).suffix) as loader:
            spec = importlib.util.spec_from_file_location(f"docker_{self.name}", self.root_dir / self._config_file_)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)

        return loader.env

    def __getstate__(self):
        return {k: v for k,v in self.__dict__.items() if k not in self._get_state_bypass_}



    def __setstate__(self, d):
        self.__dict__.update(d)
        self._env = None
        self._meta = None
        self._env_holder = None


    def refresh(self):
        self._env = self._get_env()
        self._meta = self._env.get_meta()
        self._meta.name = self.name



    @property
    def running(self):
        if not self._meta:
            self.refresh()
        return driver.is_running(self._meta.DOCKER_NAME)


    @property
    def meta(self):
        if not self._meta:
            self.refresh()
        return self._meta

    @property
    def env(self):
        if not self._meta:
            self.refresh()
            self._env_holder = self._env.get_env()
        return self._env_holder

    def check(self):
        if not self._env: self.refresh()
        return self._env.get_env()

    def create(self, env_name, values):

        envs = load_env()
        if env_name not in envs:
            print(f"Impossible de trouver l'environnement '{env_name}'", file=sys.stderr)
            exit(-1)
        env = envs[env_name]()
        content = env.create_empty_config(values)
        output = settings.app_dir / self.name / self._config_file_
        output.parent.mkdir(exist_ok=True, parents=True)
        output.write_text(content)
        output.chmod(0o750)
    def remove(self):
        shutil.rmtree(self.root_dir)


    def get_env_object(self):
        if not self._env: self.refresh()
        return self._env